import sys, time, codecs

# store these fields
VCFFields=['EMAIL', 'URL', 'TEL', 'ADR', 'BDAY', 'NOTE', 'X-KADDRESSBOOK-WANDKALENDER-NAME']

# take care of these types
VCFTelTypes=['TYPE='+t1 for t1 in ['HOME', 'WORK', 'CELL', 'FAX'] ]
VCFTelPrefTypes=[t1+';TYPE=PREF' for t1 in VCFTelTypes]             
VCFTypes=VCFTelTypes + VCFTelPrefTypes

# this one is special, not in VCFTypes
VCFTelPrefTypes.append('TYPE=PREF')

class VCard:
    "store in its __dict__-property the Fields of a single contact from the file"
    def __init__(self, Lines, quiet= False):
        self.__dict__ = {'UID': None,
                     'PrefNumber': None,    
                     'N_FamilyName': '',
                     'N_GivenName': '',
                     'N_AdditionalNames': '',
                     'N_HonoricPref': '',
                     'N_HonoricSuff': '' }
        
        for l in Lines:
            self.parseVCardLine(l, quiet)
    #__init__()

    def __str__(self):
        return 'VCard for: '#+self.getFullName()+"\n"+str(self.__dict__)
    

    def __getitem__(self, key):
        return self.__dict__[key]

    def __setitem__(self, key,val):
        self.__dict__[key] = val

    def has_key(self, k):
        return self.__dict__.has_key(k)

    def parseVCardLine(self, line, quiet):
        temp1= line.split(':',1)
        typePos= temp1[0].upper().find(';TYPE=')
        if typePos !=-1:
            FieldName= temp1[0].split(';',1)
            FieldType= FieldName[1].upper()
            FieldName= FieldName[0].upper()
        else:
            FieldName= temp1[0].upper()
            FieldType= None
            
        FieldValue=temp1[1].rstrip("\r\n")
        if FieldName == 'N':
            self.setName(FieldValue)
        elif FieldName=='UID':
            self.__dict__[FieldName]= FieldValue
        else:
            if FieldName in VCFFields:
                #print "Storing Field:", FieldName, FieldType, FieldValue
                self.storeProperty(FieldName, FieldValue, FieldType)
            else:
                if not quiet:       
                    print  "Discarding Field:", FieldName, FieldType
    # parseVCardLine
            

    def setName(self, NameString, ):
        NList = NameString.split(';')
        
        self.__dict__['N_FamilyName']= NList[0]
        self.__dict__['N_GivenName']= NList[1]
        self.__dict__['N_AdditionalNames']= NList[2]
        self.__dict__['N_HonoricPref']= NList[3]
        self.__dict__['N_HonoricSuff']= NList[4]

        self.__dict__['FN']= self.getFullName()
    # setName()


    # with spaces
    def getFullName(self):
        fn= self.__dict__['N_FamilyName']+','
        fn= fn.strip()+ ' ' + self.__dict__['N_GivenName']
        fn= fn.strip()+ ' ' + self.__dict__['N_AdditionalNames']
        fn= fn.strip()+ ' ' + self.__dict__['N_HonoricPref']
        fn= fn.strip()+ ' ' + self.__dict__['N_HonoricSuff']
        return fn.strip().rstrip(',')
    # of getFullName()


    # with ";"
    def getName(self):
        return self.__dict__['N_FamilyName']      + ';' + \
               self.__dict__['N_GivenName']       + ';' +\
               self.__dict__['N_AdditionalNames'] + ';' +\
               self.__dict__['N_HonoricPref']     + ';' +\
               self.__dict__['N_HonoricSuff']
    # of getFN()


    def getSimplifiedName(self):
        sn= self.__dict__['N_FamilyName']+','
        #sn= sn.strip()+ ' ' + self.__dict__['N_AdditionalNames'] # should it be used???
        sn= sn.strip()+ ' ' + self.__dict__['N_GivenName']
        
        return sn.strip().rstrip(',')
    # of getSimplifiedName()


    def __appendtoPropertyList__(self, key, val):
        if self.__dict__.has_key(key) and self.__dict__[key]:
            # prepend prefered, else append
            if key in VCFTelPrefTypes:
                self.__dict__[key]= [val] + self.__dict__[key]
            else:
                self.__dict__[key].append(val)
        else:
            self.__dict__[key]= [val]

        if key in VCFTelPrefTypes:
            print "setting Pref: ", key, val
            self.__dict__['PrefNumber'] = key
    # of appendtoPropertyList()

        
    def getPrefNumber():
        return self.__dict__['PrefNumber']
    
    
    def storeProperty(self, PropertyName, PropertyValue, PropertyType=None ):
        if PropertyType:
            key = PropertyName + ';' + PropertyType
        else:
            key = PropertyName
        #print PropertyName, PropertyType, PropertyValue, key 
        self.__appendtoPropertyList__(key, PropertyValue)
    # of storeProperty()


    # This is in the  RFC!
    # Break long Lines , continuation on next Lines begins with additional ' '
    def VCFbreakLine(self,fd, l, LB):
        lineLength=75    
        while len(l) > lineLength:
            # Linebreak and Space 
            fd.write(l[:lineLength]+LB+' ')
            l = l[75:]
        fd.write(l+LB)
        
        return
    #of VCFbreakLine()


    def saveContact(self, fd, LineBreak):
        LineBreak='\r\n'
        fd.write('BEGIN:VCARD'+LineBreak)

        if self.__dict__['PrefNumber']:
            pn = self.__dict__['PrefNumber']
            self.VCFbreakLine(fd, pn[0]+'TEL=Pref:'+self.__dict__[pn[1]][0], LineBreak)
            del(self.__dict__[pn[0]])

        for f in VCFFields:
            # dealing with multiple lines and lines longer 75
            for t in VCFTypes:
                key= f+';'+t
                if t.find(';TYPE=PREF') == -1:
                    if self.has_key(key+';TYPE=PREF'):
                        #dann den bevorzugten speichern
                        continue
                    
                if self.has_key(key) and self.__dict__[key]:
                    if type(self.__dict__[key]) == list:
                        for l in self.__dict__[key]:
                            self.VCFbreakLine(fd, key+':'+l, LineBreak)
                    else:
                        print "unexpected error"
                        print type(self.__dict__[key]), self.__dict__[key]
                        sys.exit(4)
            key=f
            if self.has_key(key) and self.__dict__[key]:
                if type(self.__dict__[key]) == list:
                    for l in self.__dict__[key]:
                        self.VCFbreakLine(fd, key+':'+l, LineBreak)
                else:
                    print "unexpected error"
                    print type(self.__dict__[key]), self.__dict__[key]
                    sys.exit(5)
                    
        self.VCFbreakLine(fd, 'FN:'+self.getFullName(), LineBreak)
        self.VCFbreakLine(fd, 'N:'+self.getName(), LineBreak)

        if self.__dict__['UID']:
            fd.write('UID:'+self.__dict__['UID']+LineBreak)
        fd.write('REV:'+time.strftime('%Y-%m-%dT%H:%M:%S')+LineBreak)
        fd.write('VERSION:3.0'+LineBreak)
        fd.write('END:VCARD'+LineBreak+LineBreak)
        return 
# of class VCard


class VCFCollection:
    "Stores in its Contacts-property the Contacts from a file, the entries in Contacts are Objects of class VCard"
    def __init__(self, FileName, quiet=False):
        self.FileName= FileName
        self.Contacts= []
        self.NameMap= {}
        self.UIDMap= {}
        
        self.readIt(quiet)
    # of __init__()


    def readIt(self, quiet):
        try:
            f = codecs.open(self.FileName, "rb", "utf8")
            Lines= f.readlines()
            f.close()
        except:
            print "    --> ERROR opening File: %s in VCFCollection.readIt()."%(self.FileName)
            print "        Continue with empty VCardCollection."
            Lines = []
        
        lineNr= 0
        while lineNr < len(Lines):
            while Lines[lineNr].upper().find('BEGIN:VCARD') == -1:
                lineNr+= 1
                if lineNr >= len(Lines):
                    return # when we reach the end of the file
            lineNr+=1 # step over the begin line
            
            ContactData = []
            DistributionList= False
            
            while Lines[lineNr].upper().find('END:VCARD') == -1:
                while Lines[lineNr+1][0]== ' ': #Line Continuation
                    Lines[lineNr]= Lines[lineNr].rstrip('\r\n')+Lines[lineNr+1][1:]
                    del Lines[lineNr+1]

                if Lines[lineNr].find("X-KADDRESSBOOK-DistributionList") > -1:
                    DistributionList= True

                ContactData.append(Lines[lineNr])
                lineNr+=1
                
            VC= VCard(ContactData, quiet)
            if not DistributionList: # mklug distributionlist and mklug contact had same name
                self.Contacts.append(VC)
                uid= VC.UID
                if uid !=None:
                    #print "adding UID:", uid
                    self.UIDMap[uid]= VC
                    
                key = (VC['N_GivenName']+' '+VC['N_FamilyName']).strip()
                if key=='':
                    key=VC.UID
                    
                if self.NameMap.has_key(key):
                    print VC.getFullName()
                    print '"'+key+'" already in use!'
                    print "Cowardly refusing to continue!"
                    print ContactData
                    sys.exit(1)
                else:
                    self.NameMap[key] = VC
        # end while
        
        return
    # end readIt()
        

    def deleteContact(self, UID=None, Name=None ):
        # get index in self.Contacts via UID or Name
        if UID:
            VC= self.UIDMap[ UID ]
            index = self.Contacts.index(VC)
            Name = VC.getFullName()
            
            del self.Contacs[ index ]
            del self.UIDMap[ UID ]
            if self.NameMap.has_key(Name):
                del self.NameMap[Name]
                
            return
        elif Name:
            VC= self.NameMap[ Name ]
            index = self.Contacts.index(VC)
            del self.Contacs[ index ]
            del self.NameMap[ Name ]
            
            UID = VC['UID']
            if UID and self.UIDMap.has_key( UID ) :
                del self.UIDMap[ UID ]
                
            return
    # if deleteContact()    
