# -*- coding: latin-1 -*-
import datetime

def appendYear(y,h):
    """Append y as string to the key in h. Eg. key 24.12 becomes 24.12.2007"""
    
    n = {}
    ys = '.'+str(y)
    for k in h.keys():
	n[k+ys] = h[k]
    return n
#of appendYear()


def date2Str(d):
    """Create from a datetime object d a string of the form 24.12."""
    
    return str(d.day)+'.'+str(d.month)
#of date2Str()


def addToList(dict,day, Desc):
    """Append dict[day]=Desc."""
    if dict.has_key(day):
        dict[day] = dict[day]+'/'+Desc
    else:
        dict[day] = Desc
#of addToList()


def calcEastern(Year):
    """Calculation of Eastern due to Gauss.

    Further information:
    old: http://www.ptb.de/de/org/4/44/441/oste.htm
    current: http://www.ptb.de/cms/fachabteilungen/abt4/fb-44/ag-441/darstellung-der-gesetzlichen-zeit/wann-ist-ostern.html
    And
      H. Lichtenberg,
      Zur Interpretation der Gauschen Osterformel und ihrer Ausnahmeregeln,
      Historia Mathematica 24, 441 - 444 (1997)
    """
    X  = Year
    m  = 0
    
    K  = X / 100
    M  = 15 + ((3*K+3)/4) - ((8*K+13)/25)
    S  = 2 - ((3*K+3)/4)
    A  = X % 19
    D  = (19*A+M) % 30
    R  = (D/29) + ((D/28) - (D/ 29)) * (A/11)
    OG = 21 + D - R
    SZ = 7 - (X + (X/4)+S) % 7
    OE = 7 - ((OG-SZ)% 7)

    tmp = OG+OE  # das Osterdatum als Tages des Mrz, also 32 entspricht 1. April
    if (tmp>31): # Monat erhhen, tmp=tag erniedriegen
        m = tmp / 31
        tmp -= 31
    return datetime.date(X, 3+m, tmp)
# of calcEastern())


def calcAdventTheFirst(Year):
    """Calculate the Date of the first Advent."""
    
    for t in range(27,37):
        if t > 30:
            m=12
            d= t%30
        else:
            m=11
            d=t
        tmp = datetime.date(Year,m,d)
        if tmp.weekday()==6:
            return tmp
# calcAdventTheFirst
    

def updateDicByOffset(dict, relativeTo, Offsets):
    """
    Offset is a dictionary containing Records like "Ostermontag: 0" and "Karfreitag: -3".
    relativeTo is the Date of ( in this example ) Eastern. This function add
    Ostermontag and Karfreitag to  dict with date as relativeTo + 0 and relativeTo + (-3), respectively.
    """
    for k in Offsets.keys():
        addToList(dict, date2Str(relativeTo+
                                 datetime.timedelta( Offsets[k] )), k)

    


# calculate CalendarWeek (Kalenderwoche)
def calDiff(datum, j):
    datumVierterJanuar = datetime.date(j,1,4)
    wochentagVierterJanuar = (datumVierterJanuar.weekday() )%7
    wochenDiff = ((datum - datumVierterJanuar ).days  + wochentagVierterJanuar) / 7

    return wochenDiff + 1



def calcCalWeek(d):
    jahr = d.year
    jahr+= 1

    kalWoche = calDiff(d, jahr)
    while (kalWoche < 1):
        jahr-= 1
        kalWoche = calDiff(d, jahr)

    return kalWoche
#of calcCalWeek()


def NthWeekDayInMonth(Year, Month, n, Day):
    """Calculates the nth. occurency of Day in Month.

    For Example the second sunday in May:
    Muttertag= NthWeekDayInMonth(2008,5, 2, 6)
    # where 5 means May, and 6 means Sunday and we want the second sunday in May
    #use negative n for calculation relative to the end of the month:
    Lug = NthWeekDayInMonth(2008, 3, -1, 1)
    The last (-1) Monday (1) in March.
    
    """
    
    DayInWeek={'MO': 0, 'TU':1, 'WE':2, 'TH':3, 'FR':4 , 'SA':5, 'SU':6 }
    
    if type(Day) == str:
        Day = DayInWeek[Day.upper()]
    
    if n== 0:
        raise Exception, "In calendarUtils.NthWeekDayInMonth: n == 0, which is wrong."
    elif 0< n:
        First= datetime.date(Year, Month, 1)
        diff = (Day - First.weekday()) % 7
        
        return First + datetime.timedelta(diff + (n-1) * 7)
    else: # n < 0
        if Month < 12:
            First= datetime.date(Year, (Month+1), 1)
        else:
            First= datetime.date(Year+1, 1, 1)
            
        diff = - ((First.weekday() - Day ) % 7)
        if diff == 0:
            # First is the first day of the next month, which is wrong (n < 0.
            # diff is zero if accidently First.weekday() == Day,
            # so we skip one week back:
            diff = -7

        # n is negative:
        return First + datetime.timedelta( diff + ( n +1 ) * 7)
# of NthWeekDayInMonth()
        

def NthWeekDayInMonthAsString(Year, Month, n, Day):
    """Like NthWeekDayInMonth but returns as String day.month like 11.5"""

    t= NthWeekDayInMonth(Year, Month, n, Day)
    return str(t.day)+'.'+str(t.month)
# of NthWeekDayInMonthAsString()



if __name__ == "__main__":
    print NthWeekDayInMonthAsString(2008,11,0,0)
